'use strict'

exports.up = function (knex, Promise) {
  return knex.schema.createTable('users', function (table) {
    table.increments('id').primary().index()
    table.string('email').unique().notNullable().index()
    table.string('full_name')
    table.boolean('admin').defaultTo(false)
    table.dateTime('created_at')
    table.dateTime('updated_at')
  }).createTable('organizations', function (table) {
    table.increments('id').primary().index()
    table.string('name').notNullable()
    table.string('domain').index()
    table.dateTime('created_at')
    table.dateTime('updated_at')
  }).createTable('organization_members', function (table) {
    table.increments('id').primary().index()
    table.integer('organization_id').unsigned().notNullable().index()
    table.integer('user_id').unsigned().notNullable().index()
    table.dateTime('created_at')
  }).createTable('groups', function (table) {
    table.increments('id').primary().index()
    table.integer('organization_id').unsigned().notNullable().index()
    table.string('name').notNullable()
    table.dateTime('created_at')
    table.dateTime('updated_at')
  }).createTable('group_members', function (table) {
    table.increments('id').primary().index()
    table.integer('group_id').unsigned().notNullable().index()
    table.integer('user_id').unsigned().notNullable().index()
    table.dateTime('created_at')
  }).createTable('privileges', function (table) {
    table.increments('id').primary().index()
    table.string('name').notNullable().notNullable().index()
    table.dateTime('created_at')
  }).createTable('permissions', function (table) {
    table.increments('id').primary().index()
    table.integer('privilege_id').unsigned().notNullable().index()
    table.integer('organization_id').unsigned().notNullable().index()
    table.string('resource_type')
    table.integer('resource_id').unsigned()
    table.text('expression')
    table.dateTime('created_at')
    table.dateTime('updated_at')
  })
}

exports.down = function (knex, Promise) {
}
