const _ = require('underscore')

function clean (expression) {
  // Individual Negations
  expression = expression.replace(/!($|&|\|)/g, '$1')
  // ORs
  expression = expression.replace(/(^|\()\|\|/g, '$1')
  expression = expression.replace(/\|\|($|\))/g, '$1')
  expression = expression.replace(/\|\|\|\|/g, '||')
  // ANDs
  expression = expression.replace(/(^|\()&&/g, '$1')
  expression = expression.replace(/&&($|\))/g, '$1')
  expression = expression.replace(/&&&&/g, '&&')
  // Parentheses
  expression = expression.replace(/\((!?\w\d+)\)/g, '$1')
  expression = expression.replace(/^\((.+)\)$/g, '$1')
  return expression
}

function Expression (expression) {
  this.expression = expression
}

_.extend(Expression, {
  Basic: require('./expression/basic'),
  Organic: require('./expression/organic'),

  // Get all of the variables in this expression
  variables: function (expression) {
    return expression.match(/\w\d+/g) || []
  },

  // Evaluate this expression with the variable to set to true
  evaluate: function (expression, values) {
    if (!expression || !expression.length) return false

    var variables = Expression.variables(expression)
    var args = _.map(variables, function (variable) {
      return _.include(values, variable)
    })

    var vars = variables.concat(['return ' + expression])
    return Function.apply(null, vars).apply(null, args)
  },

  // Replace a variable with this other variable
  replaceVariable: function (expression, oldVar, newVar) {
    var regEx = new RegExp(oldVar + '($|\\D)', 'g')
    return expression.replace(regEx, newVar + '$1')
  },

  // TODO: Support equations other than basic
  removeVariable: function (expression, variable) {
    expression = Expression.replaceVariable(expression, variable, '')
    expression = clean(expression)
    return expression
  }
})

_.extend(Expression.prototype, {
  evaluate: function (values) {
    return Expression.evaluate(this.expression, values)
  }
})

module.exports = Expression
