const _ = require('underscore')
const Variant = require('../variant')

// Convert these variants to an expresion
function toExpression (variants) {
  if (!variants.length) return variants.join('||')
  var expression = variants.join('||')
  if (variants.length > 1) expression = '(' + expression + ')'
  return expression
}

function Basic (includes, excludes) {
  this.includes = includes || []
  this.excludes = excludes || []
}

// Go from expression to object
Basic.parse = function (expression) {
  if (!expression) return new Basic()

  // Split includes and excludes
  var includes = []
  var excludes = []
  expression.split('&&').forEach(function (exp) {
    var variables = exp.replace(/[!()]/g, '').split('||')
    var variants = _.map(variables, Variant.fromString)
    var exclude = (exp[0] === '!')
    exclude ? excludes = variants : includes = variants
  })

  return new Basic(includes, excludes)
}

// Convert from object to expression
Basic.prototype.toString = function () {
  // Includes only
  if (!this.excludes.length) return this.includes.join('||')

  // Has excludes (may have includes)
  var includes = toExpression(this.includes)
  var excludes = toExpression(this.excludes)
  if (excludes) excludes = '!' + excludes
  return _.compact([includes, excludes]).join('&&')
}

module.exports = Basic
