const _ = require('underscore')
const Variant = require('../variant')

function Organic (organizationId, includes) {
  this.organizationId = organizationId
  this.includes = includes || []
}

// Go from expression to object
Organic.parse = function (expression) {
  // Base handlers
  if (expression instanceof Organic) return expression
  if (!expression) return new Organic()

  // Split out
  var organizationId
  var includes = []
  expression.split('&&').forEach(function (exp) {
    var variables = exp.replace(/[!()]/g, '').split('||')
    var variants = _.map(variables, Variant.fromString)
    var organizations = _.where(variants, {type: 'organization'})

    // Organization ID?
    if (organizations.length) {
      if (organizationId) throw new Error('Invalid Expression')
      if (variants.length > 1) throw new Error('Invalid Expression')
      organizationId = variants[0].id
    } else if (variants.length >= 1) {
      if (includes.length) throw new Error('Invalid Expression')
      includes = variants
    } else { // Unknown
      throw new Error('Invalid Expression')
    }
  })

  return new Organic(organizationId, includes)
}

// Determine if the first expression is a subset of the other one
Organic.isSubsetOf = function (expSub, expSup) {
  expSub = Organic.parse(expSub)
  expSub = Organic.parse(expSub)
  expSup = Organic.parse(expSup)

  // Check organization
  if (expSup.organizationId && expSub.organizationId !== expSup.organizationId) return false

  // Check variants
  var variantsSub = _.invoke(expSub.includes, 'toString')
  var variantsSup = _.invoke(expSup.includes, 'toString')
  var intersection = _.intersection(variantsSub, variantsSup)
  if (!variantsSup.length) return true
  if (!variantsSub.length) return false
  return expSub.organizationId === expSup.organizationId && intersection.length === variantsSub.length
}

// Convert these variants to an expresion
function toExpression (variants) {
  if (variants.length === 0) return
  if (variants.length === 1) return variants[0]
  return '(' + variants.join('||') + ')'
}

// Convert from object to expression
Organic.prototype.toString = function () {
  var organizationId = this.organizationId
  var includes = this.includes

  // Org and includes
  if (organizationId && includes.length) {
    return 'O' + organizationId + '&&' + toExpression(includes)
  } else if (organizationId) {
    return 'O' + organizationId
  } else if (includes.length) {
    return includes.join('||')
  } else {
    return null
  }
}

module.exports = Organic
