const _ = require('underscore')
const httpError = require('http-errors')

module.exports = function (Rbac) {
  const {Group} = Rbac

  return {
    loadById: function (req, res, next, id) {
      if (!req.currentUser) return next(httpError.Unauthorized())
      Group.findById(id).whereUserCan(req.currentUser).then(function (group) {
        if (!group) return next(httpError.NotFound('Group Not Found'))
        req.group = group
        next()
      }).catch(next)
    },

    loadFromQuery: function (req, res, next) {
      if (!req.query.group) return next()
      Group.findById(req.query.group).whereUserCan(req.currentUser).then(function (group) {
        if (!group) return next(httpError.NotFound('Group Not Found'))
        req.group = group
        next()
      }).catch(next)
    },

    index: function (req, res, next) {
      var qb = Group.query().whereUserCan(req.currentUser).whereQuery(req.query)
      if (req.organization) qb = qb.whereOrganizationId(req.organization.id)
      qb.select('groups.*').countPeople().then(function (groups) {
        res.json(groups)
      }).catch(next)
    },

    create: function (req, res, next) {
      var json = _.clone(req.body)
      json.organizationId = req.organization.id
      Group.create(json).then(function (group) {
        group.peopleCount = 0
        res.json(group)
      }).catch(next)
    },

    show: function (req, res) {
      res.json(req.group)
    },

    patch: function (req, res, next) {
      Group.update(req.group, req.body).then(function (group) {
        res.json(group)
      }).catch(next)
    },

    destroy: function (req, res, next) {
      Group.purge(req.group).then(function () {
        res.json(null)
      }).catch(next)
    },

    members: {
      create: function (req, res, next) {
        Group.Member.bulkCreate(req.group, req.body).then(function (users) {
          res.json(users)
        }).catch(next)
      },

      destroy: function (req, res, next) {
        Group.Member.removeFromGroup(req.group, req.person).then(function () {
          res.json(null)
        }).catch(next)
      }
    }
  }
}
