const Promise = require('bluebird')
const httpError = require('http-errors')

module.exports = function (Rbac) {
  const {Organization} = Rbac

  return {
    loadById: function (req, res, next, id) {
      if (!req.currentUser) return next(httpError.Unauthorized())
      Organization.findById(id).whereUserCan(req.currentUser).then(function (organization) {
        if (!organization) return next(httpError.NotFound('Organization Not Found'))
        req.organization = organization
        next()
      }).catch(next)
    },

    loadFromQuery: function (req, res, next) {
      if (!req.query.organization) return next()
      Organization.findById(req.query.organization).whereUserCan(req.currentUser).then(function (organization) {
        if (!organization) return next(httpError.NotFound('Organization Not Found'))
        req.organization = organization
        next()
      }).catch(next)
    },

    // Check to see if the current user has this permission on the organization
    can: function (privilege) {
      return function (req, res, next) {
        if (req.currentUser.admin) return next()
        Rbac.Permission.whereUser(req.currentUser, {
          privilege: privilege,
          organizationId: req.organization.id,
          resourceType: null
        }).then(function (permissions) {
          if (!permissions.length) return next(httpError.Forbidden('Access to ' + privilege + ' denied'))
          next()
        }).catch(next)
      }
    },

    index: function (req, res, next) {
      Organization.whereUserCan(req.currentUser).then(function (organizations) {
        res.json(organizations)
      }).catch(next)
    },

    create: function (req, res, next) {
      Organization.createBy(req.body).then(function (organization) {
        return Organization.loadGroups(organization)
      }).then(function (organization) {
        res.json(organization)
      }).catch(next)
    },

    show: function (req, res, next) {
      var organization = req.organization
      return Promise.all([
        Organization.loadGroups(organization),
        Organization.loadPeopleCounts(organization),
        Organization.loadPrivileges(organization, req.currentUser)
      ]).then(function () {
        res.json(organization)
      }).catch(next)
    },

    patch: function (req, res, next) {
      Organization.update(req.organization, req.body).then(function (organization) {
        res.json(organization)
      }).catch(next)
    },

    destroy: function (req, res, next) {
      Organization.removeById(req.organization.id).then(function () {
        res.json(null)
      }).catch(next)
    }
  }
}
