const httpError = require('http-errors')

module.exports = function (Rbac) {
  const {Permission} = Rbac

  return {
    loadById: function (req, res, next, id) {
      Permission.findById(id).then(function (permission) {
        if (!permission) return next(httpError.NotFound('Permission Not Found'))
        req.permission = permission
        next()
      }).catch(next)
    },

    index: function (req, res, next) {
      var organizationId = parseInt(req.query.organization, 10)

      var qb = Permission.query()
      if (organizationId) qb = qb.whereOrganizationId(organizationId)
      qb.joins('privileges').distinct('permissions.*').select('privileges.name AS privilege').then(function (permissions) {
        res.json(permissions)
      }).catch(next)
    },

    create: function (req, res, next) {
      Permission.create(req.body).then(function (permission) {
        res.json(permission)
      }).catch(next)
    },

    patch: function (req, res, next) {
      Permission.update(req.permission, req.body).then(function (permission) {
        res.json(permission)
      }).catch(next)
    },

    destroy: function (req, res, next) {
      Permission.removeById(req.permission.id).then(function () {
        res.json(null)
      }).catch(next)
    }
  }
}
