const _ = require('underscore')
const ActiveKnex = require('active-knex')

module.exports = function (knex, Rbac) {
  var Privilege = ActiveKnex.Schema.create(knex, {
    tableName: 'privileges',
    keys: ['name'],

    fields: {
      createdAt: true
    },

    joins: {
      permissions: function (qb) {
        return qb.leftJoin('permissions', 'privileges.id', 'permissions.privilege_id')
      }
    }
  })

  _.extend(Privilege, {
    findByName: function (name, trx) {
      return Privilege.transacting(trx).where({name: name}).first()
    },

    // Create a privilege by name
    createByName: function (name, trx) {
      return Privilege.findOrCreate({name: name}, trx)
    },

    // Create permissions with these names
    bulkCreate: function (name, trx) {
      var names = ActiveKnex.util.arrayify(name)
      return Privilege.transacting(trx).whereIn('name', names).then(function (privileges) {
        // Figure out which privileges are missing
        var existing = _.pluck(privileges, 'name')
        var missing = _.chain(names).difference(existing).map(function (name) {
          return {name: name}
        }).value()

        // Create the missing privileges and add to list (if necessary)
        if (!missing.length) return privileges
        return Privilege.create(missing, trx).then(function (missing) {
          return _.union(privileges, missing)
        })
      }).then(function (privileges) {
        // Ensure the privileges go out in the same name order they came in
        return _.sortBy(privileges, function (privilege) {
          return _.indexOf(names, privilege.name)
        })
      })
    }
  })

  return Privilege
}
