const _ = require('underscore')

function reducePermissions (permissions) {
  return _.chain(permissions).reduce(function (result, permission) {
    if (permission.resourceId) result.resourceIds.push(permission.resourceId)
    else result.organizationIds.push(permission.organizationId)
    return result
  }, {
    organizationIds: [],
    resourceIds: []
  }).mapObject(function (ids) {
    return _.uniq(ids).sort()
  }).value()
}

module.exports = function (Rbac, schema) {
  return {
    whereUserCan: function (user, privilege, organizationId) {
      var resourceType = schema.resourceType || schema.tableName
      return this.defer(function (qb) {
        return Rbac.Permission.whereUser(user, {
          privilege: privilege,
          organizationId: organizationId,
          resourceType: resourceType
        }).then(function (permissions) {
          qb.whereInPermissions(permissions)
        })
      })
    },

    whereInPermissions: function (permissions) {
      // Format fields
      var idField = [schema.tableName, 'id'].join('.')
      var orgIdField = [schema.tableName, 'organization_id'].join('.')

      // Reduce to ids
      var reduced = reducePermissions(permissions)
      var ids = reduced.resourceIds
      var orgIds = reduced.organizationIds

      // Both ids OR organization ids
      if (ids.length && orgIds.length) {
        return this.where(function () {
          this.whereIn(idField, ids).orWhereIn(orgIdField, orgIds)
        })
      }

      // One or the other
      return orgIds.length ? this.whereIn(orgIdField, orgIds) : this.whereIn(idField, ids)
    }
  }
}
