const _ = require('underscore')
const Promise = require('bluebird')
const ActiveKnex = require('active-knex')

module.exports = function (Rbac) {
  return {
    create: function (knex, schema) {
      var resourceType = schema.resourceType || schema.tableName
      var queries = require('./queries')(Rbac, schema)
      schema.queries = _.extend({}, queries, schema.queries)

      var Schema = ActiveKnex.Schema.create(knex, schema)
      return _.extend(Schema, {
        // Load the privileges for the user onto the model
        loadPrivileges: function (model, user) {
          if (!model || !user) return Promise.resolve(model)
          return Rbac.Permission.getPrivileges(user, {
            organizationId: model.organizationId,
            resourceType: resourceType,
            resourceId: model.id
          }).then(function (privileges) {
            model.privileges = privileges
            return model
          })
        },

        // Does the model have the privilege loaded (can do it)
        can: function (model, privilege) {
          return _.includes(model.privileges, privilege)
        },

        // Helper to start a whereUserCan query
        whereUserCan: function (user, privilege, organizationId) {
          return this.query().whereUserCan(user, privilege, organizationId)
        }
      })
    }
  }
}
