const express = require('express')

module.exports = function (Rbac) {
  // Controllers
  var auth = Rbac.controllers.auth
  var people = Rbac.controllers.people
  var groups = Rbac.controllers.groups
  var privileges = Rbac.controllers.privileges
  var organizations = Rbac.controllers.organizations
  var permissions = Rbac.controllers.permissions
  var users = Rbac.controllers.users

  // Create router
  var router = express()

  // Users
  router.param('userId', users.loadById)
  router.get('/users', auth.admin, users.index)
  router.post('/users/validate', users.validate)
  router.post('/users/reset', auth.admin, users.reset)
  router.get('/users/:userId', auth.admin, users.show)
  router.post('/users/:userId/reset', auth.admin, users.reset)

  // People
  router.param('personId', people.loadById)
  router.get('/people', auth.user, people.canIndex, people.index)
  router.patch('/people/:personId', auth.user, people.patch)
  router.delete('/people/:personId', auth.user, people.destroy)

  // Groups
  router.param('groupId', groups.loadById)
  router.get('/groups', auth.user, groups.index)
  router.get('/groups/:groupId', auth.user, groups.show)
  router.patch('/groups/:groupId', auth.user, groups.patch)
  router.delete('/groups/:groupId', auth.user, groups.destroy)

  router.post('/groups/:groupId/members', auth.user, groups.members.create)
  router.delete('/groups/:groupId/members/:personId', auth.user, groups.members.destroy)

  // Organizations
  router.param('organizationId', organizations.loadById)
  router.get('/organizations', auth.admin, organizations.index)
  router.post('/organizations', auth.admin, organizations.create)
  router.get('/organizations/:organizationId', auth.user, organizations.show)
  router.patch('/organizations/:organizationId', auth.admin, organizations.patch)
  router.delete('/organizations/:organizationId', auth.admin, organizations.destroy)
  router.post('/organizations/:organizationId/groups', auth.user, groups.create)

  router.post('/organizations/:organizationId/people', auth.user, people.create)
  router.get('/organizations/:organizationId/people/download', auth.user, people.download)
  router.post('/organizations/:organizationId/people/upload', auth.user, people.upload)

  // Privileges
  router.param('privilegeId', privileges.loadById)
  router.get('/privileges', auth.user, privileges.index)
  router.post('/privileges', auth.admin, privileges.create)
  router.patch('/privileges/:privilegeId', auth.admin, privileges.patch)
  router.delete('/privileges/:privilegeId', auth.admin, privileges.destroy)

  // Permissions
  router.param('permissionId', permissions.loadById)
  router.get('/permissions', auth.user, permissions.index)
  router.post('/permissions', auth.user, permissions.create)
  router.patch('/permissions/:permissionId', auth.user, permissions.patch)
  router.delete('/permissions/:permissionId', auth.user, permissions.destroy)

  return router
}
