/* global beforeEach describe it */
/* eslint-disable no-unused-expressions */
const request = require('supertest')
const server = require('../server')
const Factory = require('../factory')

describe('Permissions Controller', () => {
  var admin
  var organization
  var privilege
  var permission

  beforeEach(async () => {
    admin = await Factory.createUser({admin: true})
    await Factory.createUser()
    organization = await Factory.create('organization')
    await Factory.create('group', {organizationId: organization.id})
    privilege = await Factory.create('privilege')
    permission = await Factory.createPermission(privilege, organization)
  })

  describe('POST /permissions', () => {
    it('should return 401', () => {
      return request(server)
        .post('/permissions')
        .set('Accept', 'application/json')
        .expect(401)
    })

    it('should create a permission', () => {
      return request(server)
        .post('/permissions')
        .send({organizationId: organization.id, privilegeId: privilege.id})
        .set('From', admin.email)
        .set('Accept', 'application/json')
        .expect(200)
    })
  })

  describe('DELETE /permissions/:id', () => {
    it('should return 401', () => {
      return request(server)
        .del('/permissions/' + permission.id)
        .set('Accept', 'application/json')
        .expect(401)
    })

    it('should delete an application', () => {
      return request(server)
        .del('/permissions/' + permission.id)
        .set('From', admin.email)
        .set('Accept', 'application/json')
        .expect(200)
    })
  })
})
