/* global beforeEach describe it */
/* eslint-disable no-unused-expressions */
const request = require('supertest')
const {expect} = require('chai')
const server = require('../server')
const Factory = require('../factory')
const {Privilege} = require('../rbac')

describe('Privileges Controller', () => {
  var admin
  var user
  var privilege

  beforeEach(async () => {
    admin = await Factory.createUser({admin: true})
    user = await Factory.createUser()
    privilege = await Factory.create('privilege')
    await Factory.create('privilege')
  })

  describe('POST /privileges', () => {
    it('should return 401', () => {
      return request(server)
        .post('/privileges')
        .set('Accept', 'application/json')
        .expect(401)
    })

    it('should create an privilege', () => {
      return request(server)
        .post('/privileges')
        .send({name: 'name'})
        .set('From', admin.email)
        .set('Accept', 'application/json')
        .expect(200).then(function (res) {
          var json = res.body
          expect(json.name).to.equal('name')
        })
    })
  })

  describe('PATCH /privileges/:id', () => {
    it('should return 401', () => {
      return request(server)
        .patch('/privileges/' + privilege.id)
        .send({name: 'New Name'})
        .set('Accept', 'application/json')
        .expect(401)
    })

    it('should return 403', () => {
      return request(server)
        .patch('/privileges/' + privilege.id)
        .send({name: 'New Name'})
        .set('From', user.email)
        .set('Accept', 'application/json')
        .expect(403)
    })

    it('should update a user', () => {
      return request(server)
        .patch('/privileges/' + privilege.id)
        .send({name: 'name'})
        .set('From', admin.email)
        .set('Accept', 'application/json')
        .expect(200).then(function (res) {
          var json = res.body
          expect(json.name).to.equal('name')
        })
    })
  })

  describe('DELETE /privileges/:id', () => {
    it('should return 401', () => {
      return request(server)
        .del('/privileges/' + privilege.id)
        .set('Accept', 'application/json')
        .expect(401)
    })

    it('should delete an privilege', () => {
      return request(server)
        .del('/privileges/' + privilege.id)
        .set('From', admin.email)
        .set('Accept', 'application/json')
        .expect(200).then(() => {
          return Privilege.findById(privilege.id).then(function (privilege) {
            expect(privilege).to.be.undefined
          })
        })
    })
  })
})
